% Demonstrate eratoshenes construction
function eratosthenes
global d

%Latitude and longitude of pole (with measurable shadow) /deg
lat1 = 31.2;  long1 = 29.2;

%Latitude and longitude of well (sun directly overhead)
lat2 = 24.1;  long2 = 32.9;

%Default background colour
d.background_color = [0,0,0];

%Initialize file counter (this is used for multiple prints)
d.fc = 1;

%Define number of azimuthal pixels for image (which will be interpolated)
d.N = 100; 
d.highres = 0; d.Nlowres =  200;

%

%Get bitmap
import_image

%Interpolate bitmap
interp_image

%Create figure
d.fig = figure('color',d.background_color,'name','Texture map image to a sphere',...
    'numbertitle','off','KeyPressFcn',@keypressfunc,'renderer','opengl',...
    'units','normalized','position',[0.25 0.25 0.5 0.5],'InvertHardCopy', 'off');

%Texture map bitmap onto surface
texture_map_image_onto_surface
plot_eratosthenes( lat1, long1, lat2, long2 )

%%

% Plot eratosthenes construction (assume unit sphere)
function plot_eratosthenes( lat1, long1, lat2, long2 )

%Determine x,y,z vectors of positions 1 and 2
x1 = cos(lat1*pi/180)*cos(long1*pi/180); y1 = cos(lat1*pi/180)*sin(long1*pi/180); z1 = sin(lat1*pi/180);
x2 = cos(lat2*pi/180)*cos(long2*pi/180); y2 = cos(lat2*pi/180)*sin(long2*pi/180); z2 = sin(lat2*pi/180);

%Determine minor arc between positions 1 and 2
beta = acos( dot( [x1,y1,z1], [x2,y2,z2] ) );
beta = min( [beta, 2*pi/beta] );
beta_in_deg = beta*180/pi

%Define x,y,z basis vectors
xhat = [x1,y1,z1];
zhat = cross( [x1,y1,z1], [x2,y2,z2] ); zhat = zhat/sqrt( dot(zhat,zhat) );
yhat = cross( zhat, xhat );

%Define arc
a = linspace(0,beta,1000);
x = xhat(1)*cos(a) + yhat(1)*sin(a);
y = xhat(2)*cos(a) + yhat(2)*sin(a);
z = xhat(3)*cos(a) + yhat(3)*sin(a);

%Plot sphere centre to positions
plot3( 2*[0,x1],2*[0,y1],2*[0,z1] ,'r--', 'linewidth', 2  );
plot3( [0,x2],[0,y2],[0,z2] ,'r--', 'linewidth', 2  );

%Plot arc
plot3( x,y,z ,'r-', 'linewidth', 2  );

%Plot Sun ray at position  and 2
plot3( [x1,x1+x2],[y1,y1+y2],[z1,z1+z2] ,'y-', 'linewidth', 2 );
plot3( [x2,x2+x2],[y2,y2+y2],[z2,z2+z2] ,'y-', 'linewidth', 2  );

%%

% Texture map image d.I onto a surface
function texture_map_image_onto_surface
global d
disp(' Plotting surface ...')

%Generate x,y,z matrices of a surface defined in 
%terms of azimuth and elevation
elev = linspace( -pi/2,pi/2,d.N ); azi = linspace( -pi, pi, d.N*2 );
[azi,elev] = meshgrid(azi,elev);
r = ones( size(azi) );
x = r.*cos(elev).*cos(azi); y = r.*cos(elev).*sin(azi); z = r.*sin(elev);

%Clear figure and plot the texture-mapped surface
clf(d.fig); d.surface = surf( x,y,z,d.I); shading interp; axis vis3d; axis off; zoom(1.5);
alpha(0.9); hold on;
disp(' Surface plotted')

%%

% Texture map image d.I onto a surface
function update_map_image_onto_surface
global d

%Generate x,y,z matrices of a surface defined in 
%terms of azimuth and elevation
elev = linspace( -pi/2,pi/2,d.N ); azi = linspace( -pi, pi, d.N*2 );
[azi,elev] = meshgrid(azi,elev);
r = ones( size(azi) );
x = r.*cos(elev).*cos(azi); y = r.*cos(elev).*sin(azi); z = r.*sin(elev);

%Clear figure and plot the texture-mapped surface
set( d.surface,'xdata',x,'ydata',y,'zdata',z,'cdata',d.I); 

%%

%Key press handler
function keypressfunc( fig,evnt )
global d

%Get the string of the current key press
k = get( fig,'currentkey');

%Actions, depending on the key pressed...
if strcmp(k,'p') == 1
    %Print PNG image
    d.fc = d.fc + 1;
    print( gcf, [d.pathname,'bauble ',num2str(d.fc),' ',...
        strrep( d.filename, lower(d.info.Format),'png')],'-dpng','-r300' );
elseif strcmp(k,'b') == 1
    %Change background to black
    d.background_color = [0,0,0];
    set(d.fig,'color',d.background_color);
elseif strcmp(k,'w') == 1
    %Change background to white
    d.background_color = [1,1,1];
    set(d.fig,'color',d.background_color);
elseif strcmp(k,'h') == 1
    %Toggle high resolution mode
    if d.highres==0; 
        d.highres = 1;
        d.I = d.Ihighres;
        d.N = d.Nhighres;
        if d.N>2500; d.N = 2500; end
        disp(' High resolution image. Press h before rotating.')
    else
        d.highres = 0;
        d.N = d.Nlowres;     
        disp(' Low resolution image. Press h for high resolution.')
    end
    
    %Regenerate surface.
    interp_image; update_map_image_onto_surface; 
    
elseif strcmp(k,'q') == 1    
    close(gcf); %Close figure
end

%%

% Import bitmap image from a file and return a NxMx3 colour matrix d.I
% and d.Ihighres
function import_image
global d

%Chose image file to load
[d.filename, d.pathname] = uigetfile( {'*.jpg;*.bmp;*.png;*.gif;*tif',...
    'Bitmap images (*.jpg,*.bmp,*.png,*.gif,*.tif)'},...
    'Select location of image file(s)...' );
if d.filename == 0
    return
end

%Load image into an NxMx3 matrix of values. i.e. Red, Green and Blue colour
%planes. cmap is the local colormap associated with the image, and alpha is
%a matrix which defines the transparency of the image
[d.I,cmap,alphamap] = imread( [d.pathname,d.filename] );

%Get image information stored within the file
d.info = imfinfo( [d.pathname,d.filename] );
dim = size(d.I); d.Nhighres = dim(1);

%Set the colour matrix to be within the range 0...1, and a double precision
%number. Most full colour images are stored as 8 bit integers.
if numel(dim) == 2   
    if isempty(cmap);
        %Grayscale - i.e. single colour plane
        II = ones( dim(1),dim(2),3);
        II(:,:,1) = d.I(:,:); II(:,:,2) = d.I(:,:); II(:,:,3) = d.I(:,:);
        d.I = II; clear II;
    else
        %An indexed image like a GIF has a colormap rather than three R,G,B
        %colour intensities.
        d.I = 255*ind2rgb(d.I,cmap);
    end
end
d.I = double(d.I)/255;

%Flip image, as pixel(1,1) is by convention the top left rather than the
%bottom left.
d.I = flipdim(d.I,1);

%Store separate full resolution image
d.Ihighres = d.I;

%%

%Interpolate bitmap image to a grid 2*d.N in azimuth and d.N in elevation
function interp_image
global d
disp(' Interpolating image ...' )

%Define index vectors for height and width of image d.II
dim = size(d.I); heightI = dim(1); widthI = dim(2); 
w = 1:widthI; h = 1:heightI;

%Define index vectors for height and width of output image
W = linspace(1,widthI,d.N*2 ); H = linspace(1,heightI,d.N );

%Meshgrid these
[h,w] = meshgrid(h,w); [H,W] = meshgrid(H,W);

%Create R,G,B matricies from image matrix I.
R = ones(heightI,widthI); G = ones(heightI,widthI); B = ones(heightI,widthI);
R(:,:) = d.I(:,:,1); G(:,:) = d.I(:,:,2); B(:,:) = d.I(:,:,3);

%Interpolate these colour matricies
R = interp2(h,w,R.',H,W); G = interp2(h,w,G.',H,W); B = interp2(h,w,B.',H,W);
clear h H w W

%Initialise output array
dim = size(R.'); 
II = zeros(dim(1),dim(2),3); 
II(:,:,1) = R.'; II(:,:,2) = G.'; II(:,:,3) = B.';
clear R G B

%Assign output
d.I = II; clear II;
disp(' Image interpolated' )

%End of code