%greatcircle
% Program which generates a unit sphere with a texture mapped Earth
% surface. Overlaid is an arc of a great circle representing the shortest
% distance between two points defined on the surface of the sphere.
% 
% a,z modifies the latitude (up,down) of position 1
% s,d modifies the longitude (left,right) of position 1
%
% up arrow,down arrow modifies the latitude (up,down) of position 2
% left arrow,right arow modifies the longitude (left,right) of position 2
%
% Press p to print and q to quit.
%
% LAST UPDATED by Andy French January 2017

function greatcircle

%% Set Defaults %%

%Define global variable d to share between functions
global d

%Radius of Earth in km
d.REarth_km = 6371;

%Fontsize
d.fsize = 16;

%Initial view location in [long, lat]
d.view = [140,35];

%Initial latitude /radians of position A
d.lat1 = pi/10;

%Initial longitude /radians of position B
d.long1 = pi/3;

%Initial latitude /radians of position A
d.lat2 = -pi/3;

%Initial longitude /radians of position B
d.long2 = 0.7*pi;

%Latitude and longitude step sizes /radians
d.latstep = pi/180;
d.longstep = pi/180;

%Number of points for circle
d.N = 300;

%Increments for lines of longitude and latitude
d.lat_increment = (1/20)*pi;
d.long_increment = (1/20)*pi;

%

%% Initial setup %%

%Create figure
d.fig = figure('name','spherecircle','color',[1 1 1],'KeyPressFcn',@keypress,...
    'units','normalized','position',[0,0,1,1],'numbertitle','off');

%Load Earth bitmap
I = imread('earth_reallylowres.jpg');
s = size(I);
I = double(I)/255;
I = flipdim(I,1);

%Define x,y,z coordinates of unit sphere
lat = linspace(-pi/2,pi/2,s(1));
long = linspace(-pi,pi,s(2));
[long,lat] = meshgrid(long,lat);
x = cos(lat).*cos(long);
y = cos(lat).*sin(long);
z = sin(lat);

%Plot sphere and texture-map Earth onto it
earth = surf(x,y,z,I);
axis vis3d;
shading interp;
hold on;
grid on;
xlabel('x','fontsize',d.fsize);
ylabel('y','fontsize',d.fsize);
zlabel('z','fontsize',d.fsize);
set(gca,'fontsize',d.fsize);
view(d.view)
axis manual

%Lines of longitude
long = 0 : d.long_increment : 2*pi - 2*pi/10;
for n=1:length(long)
    [x,y,z] = sphere_circle( 0, long(n), 1, pi/2, 300);
    plot3(x,y,z,'w-');
end

%

%Lines of latitude
lat = 0 : d.lat_increment : pi/2;
for n=1:length(lat)
    [x,y,z] = sphere_circle( pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
    [x,y,z] = sphere_circle( -pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
end

%

%Determine coordinates of great circle arc between points on sphere
[x,y,z,arc_length,xgc,ygc,zgc] = great_circle_arc( d.lat1, d.long1, d.lat2, d.long2, d.N );
d.great_circle = plot3(xgc,ygc,zgc,'g--');
d.arc = plot3(x,y,z,'g-','linewidth',2);

%Plot points
[x,y,z] = latlong2xyz( d.lat1, d.long1, 1 );
d.p1 = plot3( x,y,z, 'g*','markersize',18 );
[x,y,z] = latlong2xyz( d.lat2, d.long2, 1 );
d.p2 = plot3( x,y,z, 'r*','markersize',18 );

%Define title
d.title = {['latA = ',...
    num2str(180*d.lat1/pi,3),'^o, longA = ',num2str(d.long1*180/pi,3),'^o, latB = ',...
    num2str(180*d.lat2/pi,3),'^o, longB = ',num2str(d.long2*180/pi,3),'^o',...
    ', arc length = ',num2str(arc_length,3) ],...
    ['Equivalent Earth arc length = ',num2str(arc_length*d.REarth_km,5),'km']};
title( d.title,'fontsize',d.fsize,'position',[8.54,10.28,11.0]);

%%

%latlong2xyz
% Converts latitude and longitude coordinates (radians) to x,y,z
% coordinates on the surface of a sphere of radius r
function [x,y,z] = latlong2xyz( lat, long, r )
x = r.*cos(lat).*cos(long);
y = r.*cos(lat).*sin(long);
z = r.*sin(lat);

%%

%latlong2tangentxyz
% Determines x,y,z unit vectors in local tangent plane to sphere surface.
% z is a radial, y points to 'North' and x completes a right handed set.
function [xhat,yhat,zhat] = latlong2tangentxyz( lat, long )
xhat = [ -sin(long) ; cos(long) ; 0 ];
yhat = [ -cos(long)*sin(lat) ; -sin(long)*sin(lat) ; cos(lat) ];
zhat = [cos(long)*cos(lat) ; sin(long)*cos(lat) ; sin(lat) ];

%%

%spherecircle
% Defines Cartesian coordinates of a circle on the surface of a sphere.
function [x,y,z] = sphere_circle( lat, long, R, alpha, N)

%Define circle polar angle
g = linspace(0,2*pi,N);

%Define unit vectors of tangent plane
[xhat,yhat,zhat] = latlong2tangentxyz( lat, long );

%Define circle coordinates as a vector equation
xhat = repmat(xhat,[1,N]);
yhat = repmat(yhat,[1,N]);
zhat = repmat(zhat,[1,N]);
r = R*sin(alpha)*( [cos(g) ; cos(g) ; cos(g) ].*xhat +...
    [sin(g) ; sin(g) ; sin(g) ].*yhat ) + R*cos(alpha)*zhat;

%Determine outputs
x = r(1,:);
y = r(2,:);
z = r(3,:);

%%

%Coordinates of section of great circle between two points on a unit
%sphere. Also returns arc length, and coordinates of the full great circle
%which passes through both points.
function [x,y,z,arc_length,xgc,ygc,zgc] = great_circle_arc( lat1, long1, lat2, long2, N )

%Form x,y,z vectors of points on unit sphere
[x1,y1,z1] = latlong2xyz( lat1, long1, 1 );
[x2,y2,z2] = latlong2xyz( lat2, long2, 1 );

%Determine (acute) angle between them
beta = acos( dot( [x1,y1,z1],[x2,y2,z2] ) );
beta = min( [beta, 2*pi-beta] );

%Hence determine arc length bewen points
arc_length = beta;

%Form a unit vector which is perpendicular to both points
zhat = cross( [x1;y1;z1],[x2;y2;z2] );
zhat = zhat/sqrt(sum(zhat.*zhat));

%Determine lat and long that zhat points at
lat = asin( zhat(3) );
long = atan2( zhat(2), zhat(1) );

%Great circle which passes through points
[xgc,ygc,zgc] = sphere_circle( lat, long, 1, pi/2, N);

%Determine xhat and yhat coordinates based on r1 and zhat
xhat = [x1;y1;z1];
yhat = cross( zhat, xhat );

%Determine x,y,z coordinates of arc
g = linspace(0,beta,N);
g = repmat(g,[3,1]);
xhat = repmat( xhat, [1,N]);
yhat = repmat( yhat, [1,N]);
r = cos(g).*xhat + sin(g).*yhat;
x = r(1,:);
y = r(2,:);
z = r(3,:);

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'p')==1
    
    %Print .png file
    [lat1,long1] = latlongwrap(d.lat1,d.long1);
    [lat2,long2] = latlongwrap(d.lat2,d.long2);
    filename = ['Great circle ',num2str(180*lat1/pi),' long=',num2str(long1*180/pi),...
        num2str(180*lat2/pi),' long=',num2str(long2*180/pi)];
    filename = strrep( filename,'.','p' );
    print(gcf,[filename,'.png'],'-dpng','-r300');
    
    elseif strcmp(get(fig,'currentkey'),'a')==1
    
    %Increase circle normal latitude
    d.lat1 = d.lat1 + d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'z')==1
    
    %Decrease circle normal latitude
    d.lat1 = d.lat1 - d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'s')==1
    
    %Increase circle normal longitude
    d.long1 = d.long1 - d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'d')==1
    
    %Increase circle normal longitude
    d.long1 = d.long1 + d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'uparrow')==1
    
    %Increase circle normal latitude
    d.lat2 = d.lat2 + d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    
    %Decrease circle normal latitude
    d.lat2 = d.lat2 - d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1
    
    %Increase circle normal longitude
    d.long2 = d.long2 - d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    
    %Increase circle normal longitude
    d.long2 = d.long2 + d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'q')==1
    
    %Close figure
    close(d.fig);
    return
end

%Update circle calculations and update plot
[x,y,z,arc_length,xgc,ygc,zgc] = great_circle_arc( d.lat1, d.long1, d.lat2, d.long2, d.N );
set( d.great_circle, 'xdata',xgc,'ydata',ygc,'zdata',zgc );
set( d.arc, 'xdata',x,'ydata',y,'zdata',z);

%Plot points
[x,y,z] = latlong2xyz( d.lat1, d.long1, 1 );
set( d.p1, 'xdata',x,'ydata',y,'zdata',z);
[x,y,z] = latlong2xyz( d.lat2, d.long2, 1 );
set( d.p2, 'xdata',x,'ydata',y,'zdata',z);

%Update title
[lat1,long1] = latlongwrap(d.lat1,d.long1);
[lat2,long2] = latlongwrap(d.lat2,d.long2);
d.title = {['latA = ',...
    num2str(180*lat1/pi,3),'^o, longA = ',num2str(long1*180/pi,3),'^o, latB = ',...
    num2str(180*lat2/pi,3),'^o, longB = ',num2str(long2*180/pi,3),'^o',...
    ', arc length = ',num2str(arc_length,3) ],...
    ['Equivalent Earth arc length = ',num2str(arc_length*d.REarth_km,5),'km']};
title(d.title,'fontsize',d.fsize,'position',[8.54,10.28,11.0]);

%%

%Wrap lat, long into correct range
function [lat,long] = latlongwrap(lat,long)

%Remove multiples of 2*pi
lat = rem( lat, 2*pi);
long = rem( long, 2*pi);

%Give correct latitudes
if lat > 0
    if ( lat <= pi ) && ( lat > pi/2 )
        lat = pi - lat;
        long = long + pi;
    elseif ( lat > pi ) && ( lat < 3*pi/2 )
        lat = - (lat - pi );
        long = long + pi;
    elseif ( lat >= 3*pi/2 ) && ( lat < 2*pi )
        lat = - (2*pi - lat );
    end
else
    if ( lat <= -pi/2 ) && ( lat > -pi )
        lat = - (pi + lat );
        long = long + pi;
    elseif ( lat <= -pi ) && ( lat > -3*pi/2 )
        lat = - (pi + lat) ;
        long = long + pi;
    elseif    ( lat > -2*pi ) && ( lat <= -3*pi/2 )
        lat = 2*pi + lat;
    end
end

%Give correct longitudes
if long > pi
    long = -(2*pi - long );
elseif long < -pi
    long = 2*pi + long;
end
    
%End of code