% Program which generates a unit sphere with a texture mapped Earth
% surface. On this is overlaid a circle, drawn on the spherical surface.
% Left and right arrows control the longitude of the circle normal, and up
% and down arrow keys control the circle latitude. Keys a and z grow or
% shrink the opening angle of the circle. Minimum is fixed at zero
% (obviously!) and maximum is 180 degrees.
%
% Press p to print and q to quit.
function spherecircle
global d

%Fontsize
d.fsize = 16;

%Initial view location in [long, lat]
d.view = [140,35];

%Default half angle of circle /radians
d.alpha = pi/6;

%Angle step /radians
d.alphastep = pi/180;

%Initial latitude /radians of circle normal
d.lat = pi/10;

%Initial longitude /radians of circle
d.long = pi/3;

%Latitude and longitude step sizes /radians
d.latstep = pi/30;
d.longstep = pi/30;

%Number of points for circle
d.N = 300;

%Increments for lines of longitude and latitude
d.lat_increment = (1/20)*pi;
d.long_increment = (1/20)*pi;

%

%% Initial setup %%

%Create figure
d.fig = figure('name','spherecircle','color',[1 1 1],'KeyPressFcn',@keypress,...
    'units','normalized','position',[0,0,1,1],'numbertitle','off');

%Load Earth bitmap
I = imread('earth_reallylowres.jpg');
s = size(I);
I = double(I)/255;
I = flipdim(I,1);

%Define x,y,z coordinates of unit sphere
lat = linspace(-pi/2,pi/2,s(1));
long = linspace(-pi,pi,s(2));
[long,lat] = meshgrid(long,lat);
x = cos(lat).*cos(long);
y = cos(lat).*sin(long);
z = sin(lat);

%Plot sphere and texture-map Earth onto it
earth = surf(x,y,z,I);
axis vis3d;
shading interp;
hold on;
grid on;
xlabel('x','fontsize',d.fsize);
ylabel('y','fontsize',d.fsize);
zlabel('z','fontsize',d.fsize);
set(gca,'fontsize',d.fsize);
view(d.view)
axis manual

%Lines of longitude
long = 0 : d.long_increment : 2*pi - 2*pi/10;
for n=1:length(long)
    [x,y,z] = sphere_circle( 0, long(n), 1, pi/2, 300);
    plot3(x,y,z,'w-');
end

%

%Lines of latitude
lat = 0 : d.lat_increment : pi/2;
for n=1:length(lat)
    [x,y,z] = sphere_circle( pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
    [x,y,z] = sphere_circle( -pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
end

%

%Define spherical circle
[x,y,z] = sphere_circle( d.lat, d.long, 1, d.alpha, d.N);
d.circle = plot3(x,y,z,'y-','linewidth',2);

%

%Plot normal to circle
[x,y,z] = latlong2xyz( d.lat, d.long, 1.5 );
d.normal = plot3( [0,x],[0,y],[0,z],'g-','linewidth',2 );


%Define title
d.title = ['Sphere circle: \alpha = ',num2str(180*d.alpha/pi,3),'^o, lat = ',...
    num2str(180*d.lat/pi),'^o, long = ',num2str(d.long*180/pi),'^o'];
title( d.title,'fontsize',d.fsize,'position',[8.54,10.28,11.0]);

%%

%latlong2xyz
% Converts latitude and longitude coordinates (radians) to x,y,z
% coordinates on the surface of a sphere of radius r
function [x,y,z] = latlong2xyz( lat, long, r )
x = r.*cos(lat).*cos(long);
y = r.*cos(lat).*sin(long);
z = r.*sin(lat);

%%

%latlong2tangentxyz
% Determines x,y,z unit vectors in local tangent plane to sphere surface.
% z is a radial, y points to 'North' and x completes a right handed set.
function [xhat,yhat,zhat] = latlong2tangentxyz( lat, long )
xhat = [ -sin(long) ; cos(long) ; 0 ];
yhat = [ -cos(long)*sin(lat) ; -sin(long)*sin(lat) ; cos(lat) ];
zhat = [cos(long)*cos(lat) ; sin(long)*cos(lat) ; sin(lat) ];

%%

%spherecircle
% Defines Cartesian coordinates of a circle on the surface of a sphere.
function [x,y,z] = sphere_circle( lat, long, R, alpha, N)

%Define circle polar angle
g = linspace(0,2*pi,N);

%Define unit vectors of tangent plane
[xhat,yhat,zhat] = latlong2tangentxyz( lat, long );

%Define circle coordinates as a vector equation
xhat = repmat(xhat,[1,N]);
yhat = repmat(yhat,[1,N]);
zhat = repmat(zhat,[1,N]);
r = R*sin(alpha)*( [cos(g) ; cos(g) ; cos(g) ].*xhat +...
    [sin(g) ; sin(g) ; sin(g) ].*yhat ) + R*cos(alpha)*zhat;

%Determine outputs
x = r(1,:);
y = r(2,:);
z = r(3,:);

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'p')==1
    
    %Print .png file
    [lat,long] = latlongwrap(d.lat,d.long);
    filename = ['Sphere cirlce alpha=',num2str(180*d.alpha/pi,3),' lat=',...
        num2str(180*lat/pi,3),' long=',num2str(long*180/pi,3)];
    filename = strrep( filename,'.','p' );
    print(gcf,[filename,'.png'],'-dpng','-r300');
    
elseif strcmp(get(fig,'currentkey'),'z')==1
    
    %Reduce size of circle angle
    d.alpha = d.alpha - d.alphastep;
    if d.alpha < 0
        d.alpha = 0;
    end
    
elseif strcmp(get(fig,'currentkey'),'a')==1
    
    %Increase size of circle angle
    d.alpha = d.alpha + d.alphastep;
    if d.alpha > pi/2
        d.alpha = pi/2;
    end
    
elseif strcmp(get(fig,'currentkey'),'uparrow')==1
    
    %Increase circle normal latitude
    d.lat = d.lat + d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    
    %Decrease circle normal latitude
    d.lat = d.lat - d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1
    
    %Increase circle normal longitude
    d.long = d.long - d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    
    %Increase circle normal longitude
    d.long = d.long + d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'q')==1
    
    %Close figure
    close(d.fig);
    return
end

%Update circle calculations and update plot
[x,y,z] = sphere_circle( d.lat, d.long, 1, d.alpha, d.N);
set(d.circle,'xdata',x,'ydata',y,'zdata',z );
[x,y,z] = latlong2xyz( d.lat, d.long, 1.5 );
set(d.normal,'xdata',[0,x],'ydata',[0,y],'zdata',[0,z] );

%Update title
[lat,long] = latlongwrap(d.lat,d.long);
d.title = ['Sphere circle: \alpha = ',num2str(180*d.alpha/pi,3),'^o, lat = ',...
    num2str(180*lat/pi,3),'^o, long = ',num2str(long*180/pi,3),'^o'];
title(d.title,'position',[8.54,10.28,11.0]);

%%

%Wrap lat, long into correct range
function [lat,long] = latlongwrap(lat,long)

%Remove multiples of 2*pi
lat = rem( lat, 2*pi);
long = rem( long, 2*pi);

%Give correct latitudes
if lat > 0
    if ( lat <= pi ) && ( lat > pi/2 )
        lat = pi - lat;
        long = long + pi;
    elseif ( lat > pi ) && ( lat < 3*pi/2 )
        lat = - (lat - pi );
        long = long + pi;
    elseif ( lat >= 3*pi/2 ) && ( lat < 2*pi )
        lat = - (2*pi - lat );
    end
else
    if ( lat <= -pi/2 ) && ( lat > -pi )
        lat = - (pi + lat );
        long = long + pi;
    elseif ( lat <= -pi ) && ( lat > -3*pi/2 )
        lat = - (pi + lat) ;
        long = long + pi;
    elseif    ( lat > -2*pi ) && ( lat <= -3*pi/2 )
        lat = 2*pi + lat;
    end
end

%Give correct longitudes
if long > pi
    long = -(2*pi - long );
elseif long < -pi
    long = 2*pi + long;
end
    
%End of code